from ..translated import translated as переводы
from ..argparser import аргпарсер
import os
from ..codegen_utils import БазовыйГенератор, получить_парсеры, загрузить, transform

здесь = os.path.abspath(os.path.dirname(__file__))
(
    parser,
    вычислительный_парсер,
) = получить_парсеры(os.path.join(здесь, "birp-rev.lark"))
переводы = {value: key for key, value in переводы.items()}


class Кодогенератор(БазовыйГенератор):
    вычислительный_парсер = вычислительный_парсер
    переводы = переводы
    keys = {
        "if": "если",
        "else": "иначе",
        "elif": "иначеесли",
        "while": "пока",
        "for": "для",
        "try": "попробовать",
        "except": "кроме",
        "finally": "финально",
        "from": "из",
        "in": "в",
        "import": "подключить",
        "as": "как",
        "pass": "пропустить",
        "continue": "продолжить",
        "break": "остановить",
        "return": "вернуть",
        "def": "объявить",
        "raise": "бросить",
        "assert": "убедиться",
        "class": "класс",
        "yield": "выдать",
        "await": "ожидать",
        "async": "асинхронно",
        "lambda": "лямбда",
        "None": "Ничего",
        "True": "Да",
        "False": "Нет",
        "and": "и",
        "or": "или",
        "not": "не",
        "is": "есть",
        "del": "удалить",
        "match": "соответствие",
        "case": "случай",
        "global": "глобально",
        "nonlocal": "нелокально",
        "with": "с",
        "orig_in": "in",
        "orig_is": "is",
        "orig_not": "not",
    }


преобразователь = Кодогенератор()


def main():
    args = аргпарсер.parse_args()
    if args.переводы:
        with open(args.переводы) as file:
            переводы_пользователя = загрузить(file)

        переводы.update(переводы_пользователя)
    if args.файлы:
        for arg in args.файлы:
            имя_ввода = arg
            with open(имя_ввода) as file:
                code = file.read()

            имя_вывода = имя_ввода.replace(".py", ".birp")
            (
                result,
                tree,
            ) = transform(code, parser, преобразователь, имя_ввода)
            if result:
                if args.tree:
                    with open(имя_вывода + ".tree", "w") as file:
                        file.write(tree.pretty())

                with open(имя_вывода, "w") as file:
                    file.write(result)

                print(f"{имя_ввода} -> {имя_вывода}")
            else:
                print(f"{имя_ввода} невозможно преобразовать в {имя_вывода}")
    else:
        while True:
            print("Интерактивный Реверсивный Борп!")
            code = input(">>> ")
            result = str(
                transform(code, вычислительный_парсер, преобразователь, "<ввод>")
            )
            print(f"=> {result}")
            print(f"= {repr(eval(result))}")


if __name__ == "__main__":
    main()



"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of birp package.
"""

__title__ = "birp"
__description__ = "birp — большой русский питон (BIg Russian Python)"
__version__ = "0.2.0"
__author__ = "evtn"
__author_email__ = "g@evtn.ru"
__license__ = "MIT"
__url__ = "https://github.com/evtn/birp"



from argparse import ArgumentParser as ПарсерАргументов

аргпарсер = ПарсерАргументов(
    description="birp - большой русский питон. Позволяет писать программы на русском языке, как настоящий патриот!!!!", 
    prog="birp[.reverse]"
)
аргпарсер.add_argument(
    "-ф", "--файлы", "-f", "--files", nargs="+", help="Список файлов, которые необъодимо перевести"
)
аргпарсер.add_argument(
    "-п",
    "--переводы",
    help="ЯСНО*-файл с дополнительными переводами имён (должен содержать объект вида {русское_имя: английское_имя})\n*ЯваСценарная Нотация Объектов",
)
аргпарсер.add_argument(
    "-д",
    "--дерево",
    dest="tree",
    action="store_true",
    help="Вывод дерева в отдельный файл",
)



#!/usr/bin/env python3

import os
from setuptools import setup


# get key package details from birp/__version__.py
about = {}  # type: ignore
here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'birp', '__version__.py')) as f:
    exec(f.read(), about)

# load the README file and use it as the long_description for PyPI
with open('README.md', 'r', encoding="utf-8") as f:
    readme = f.read()

# package configuration - for reference see:
# https://setuptools.readthedocs.io/en/latest/setuptools.html#id9
setup(
    name=about['__title__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    packages=['birp', 'birp.reverse', 'birp.translate'],
    install_requires=["lark"],
    package_data={"": ["*.lark", "*.birp"]},
    include_package_data=True,
    python_requires=">=3.6.*",
    license=about['__license__'],
    download_url=f"https://github.com/evtn/birp/archive/v{about['__version__']}.tar.gz",
    zip_safe=False,
    # https://pypi.org/classifiers/
    classifiers=[ 
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'   
    ],
    keywords='python russian translation'
)



magic = {
    "инверт": "invert",
    "нег": "neg",
    "поз": "pos",
    "абс": "abs",
    "сложение": "add",
    "и": "and",
    "деление": "div",
    "делост": "divmod",
    "целдел": "floordiv",
    "лсдвиг": "lshift",
    "ост": "mod",
    "умножение": "mul",
    "или": "or",
    "степень": "pow",
    "псдвиг": "rshift",
    "вычитание": "sub",
    "правдел": "truediv",
    "искл_или": "xor",
    "содержит": "contains",
    "рсложение": "radd",
    "ри": "rand",
    "рделение": "rdiv",
    "рделост": "rdivmod",
    "рцелдел": "rfloordiv",
    "рлсдвиг": "rlshift",
    "рост": "rmod",
    "румножение": "rmul",
    "рили": "ror",
    "рстепень": "rpow",
    "рпсдвиг": "rrshift",
    "рвычитание": "rsub",
    "рправдел": "rtruediv",
    "рискл_или": "rxor",
    "нмсложение": "iadd",
    "нми": "iand",
    "нмделение": "idiv",
    "нмцелдел": "ifloordiv",
    "нмлсдвиг": "ilshift",
    "нмост": "imod",
    "нмумножение": "imul",
    "нмили": "ior",
    "нмстепень": "ipow",
    "нмпсдвиг": "irshift",
    "нмвычитание": "isub",
    "нмправдел": "itruediv",
    "нмискл_или": "ixor",
    "равно": "eq",
    "болравно": "ge",
    "больше": "gt",
    "менравно": "le",
    "меньше": "lt",
    "неравно": "ne",
    "сравн": "cmp",
    "рсравн": "rcmp",
    "логический": "bool",
    "строка": "str",
    "ненулевое": "nonzero",
    "юникод": "unicode",
    "байты": "bytes",
    "комплексное": "complex",
    "нецелое": "float",
    "индекс": "index",
    "целое": "int",
    "округлить": "round",
    "длинное": "long",
    "формат": "format",
    "длина": "len",
    "подсказка_длины": "length_hint",
    "хэш": "hash",
    "представление": "repr",
    "развернутый": "reversed",
    "конвертация": "coerce",
    "шестн": "hex",
    "восьм": "oct",
    "путьфс": "fspath",
    "итер": "iter",
    "след": "next",
    "аитер": "aiter",
    "аслед": "anext",
    "ожидать": "await",
    "удалатр": "delattr",
    "удалэлем": "delitem",
    "удалсрез": "delslice",
    "полатр": "getattr",
    "полатрибут": "getattribute",
    "полэлем": "getitem",
    "полсрез": "getslice",
    "устатр": "setattr",
    "устэлем": "setitem",
    "устсрез": "setslice",
    "свойства": "dir",
    "отсутствует": "missing",
    "войти": "enter",
    "выйти": "exit",
    "авойти": "aenter",
    "авыйти": "aexit",
    "вызвать": "call",
    "удал": "del",
    "инициализация": "init",
    "новый": "new",
    "инициализация_подкласса": "init_subclass",
    "проверкаэкземпляра": "instancecheck",
    "проверкаподкласса": "subclasscheck",
    "полноваргументы": "getnewargs",
    "полноваргументы_экс": "getnewargs_ex",
    "полсостояние": "getstate",
    "устсостояние": "setstate",
    "свести": "reduce",
    "севсти_экс": "reduce_ex",
    "удалить": "delete",
    "получить": "get",
    "установить": "set",
    "установить_имя": "set_name",
    "подклассы": "subclasses",
    "после_инициализации": "post_init",
    "класс_полэлем": "class_getitem",
    "прм_записи": "mro_entries",
    "класс": "class",
    "словарь": "dict",
    "документация": "doc",
    "модуль": "module",
    "имя": "name",
    "всё": "all",
    "файл": "file",
    "пакет": "package",
    "аннотации": "annotations",
    "замыкание": "closure",
    "код": "code",
    "умолчания": "defaults",
    "функ": "func",
    "глобальные": "globals",
    "клумолчания": "kwdefaults",
    "сам": "self",
    "квалимя": "qualname",
    "базы": "bases",
    "подготовить": "prepare",
    "слоты": "slots",
    "метакласс": "metaclass",
    "прм": "mro",
}
magic = {f"__{k}__": f"__{v}__" for k, v in magic.items()}

types = {
    "строка": "str",
    "целое": "int",
    "нецелое": "float",
    "логический": "bool",
    "Многоточие": "Ellipsis",
    "срез": "slice",
    "список": "list",
    "массивбайтов": "bytearray",
    "байты": "bytes",
    "комплексное": "complex",
    "словарь": "dict",
    "морозноемножество": "frozenset",
    "память": "memoryview",
    "объект": "object",
    "множество": "set",
    "кортеж": "tuple",
}

builtin_functions = {
    "__подключить__": "__import__",
    "все": "all",
    "абс": "abs",
    "любой": "any",
    "бин": "bin",
    "вызываемое": "callable",
    "символ": "chr",
    "метод_класса": "classmethod",
    "собрать": "compile",
    "удалатр": "delattr",
    "свойства": "dir",
    "делост": "divmod",
    "пронумеровать": "enumerate",
    "вычислить": "eval",
    "фильтр": "filter",
    "формат": "format",
    "полатр": "getattr",
    "глобальные": "globals",
    "естьатр": "hasattr",
    "хэш": "hash",
    "помощь": "help",
    "шестн": "hex",
    "айди": "id",
    "ввод": "input",
    "есть_экземпляр": "isinstance",
    "есть_подкласс": "issubclass",
    "итер": "iter",
    "длина": "len",
    "местные": "locals",
    "применить": "map",
    "максимум": "max",
    "минимум": "min",
    "след": "next",
    "восьм": "oct",
    "открыть": "open",
    "код_символа": "ord",
    "степень": "pow",
    "свойство": "property",
    "диапазон": "range",
    "представление": "repr",
    "развернутый": "reversed",
    "округлить": "round",
    "устатр": "setattr",
    "сортированный": "sorted",
    "статическийметод": "staticmethod",
    "сумма": "sum",
    "вышестоящий": "super",
    "тип": "type",
    "переменные": "vars",
    "комбинировать": "zip",
    "точкаостановки": "breakpoint",
    "выполнить": "exec",
    "вывод": "print",
}

errors = {
    "ОшибкаАрифметики": "ArithmeticError",
    "ОшибкаПроверки": "AssertionError",
    "ОшибкаАтрибута": "AttributeError",
    "ОшибкаБлокирующегоВводаВывода": "BlockingIOError",
    "ОшибкаСломаннойТрубы": "BrokenPipeError",
    "ОшибкаБуфера": "BufferError",
    "ОшибкаДочернегоПроцесса": "ChildProcessError",
    "ОшибкаРазорванногоСоединения": "ConnectionError",
    "ОшибкаОтклоненногоСоединения": "ConnectionAbortedError",
    "ОшибкаСброшенногоСоединения": "ConnectionRefusedError",
    "ОшибкаСоединения": "ConnectionResetError",
    "ОшибкаКонцаФайла": "EOFError",
    "ОшибкаОкружения": "EnvironmentError",
    "ОшибкаСуществованияФайла": "FileExistsError",
    "ОшибкаНесуществующегоФайла": "FileNotFoundError",
    "ОшибкаПлавающейТочки": "FloatingPointError",
    "ОшибкаПрерывания": "InterruptedError",
    "ОшибкаВводаВывода": "IOError",
    "ОшибкаДиректории": "IsADirectoryError",
    "ОшибкаПодключения": "ImportError",
    "ОшибкаОтступов": "IndentationError",
    "ОшибкаИндекса": "IndexError",
    "ОшибкаКлюча": "KeyError",
    "ОшибкаПоиска": "LookupError",
    "ОшибкаПамяти": "MemoryError",
    "ОшибкаИмени": "NameError",
    "ОшибкаНеДиректории": "NotADirectoryError",
    "ОшибкаНереализованности": "NotImplementedError",
    "ОшибкаОС": "OSError",
    "ОшибкаПереполнения": "OverflowError",
    "ОшибкаРазрешения": "PermissionError",
    "ОшибкаПоискаПроцесса": "ProcessLookupError",
    "ОшибкаСсылки": "ReferenceError",
    "ОшибкаВыполнения": "RuntimeError",
    "ОшибкаСтандартная": "StandardError",
    "ОшибкаСинтаксиса": "SyntaxError",
    "ОшибкаСистемы": "SystemError",
    "ОшибкаТабов": "TabError",
    "ОшибкаТаймаута": "TimeoutError",
    "ОшибкаТипа": "TypeError",
    "ОшибкаНесвязаннойЛокальнойПеременной": "UnboundLocalError",
    "ОшибкаКодированияЮникода": "UnicodeError",
    "ОшибкаДекодированияЮникода": "UnicodeEncodeError",
    "ОшибкаТрансляцииЮникода": "UnicodeDecodeError",
    "ОшибкаЮникода": "UnicodeTranslateError",
    "ОшибкаЗначения": "ValueError",
    "ОшибкаVMS": "VMSError",
    "ОшибкаВиндовс": "WindowsError",
    "ОшибкаДеленияНаНоль": "ZeroDivisionError",
    "Предупреждение": "Warning",
    "ПредупреждениеОжидающегосяУстаревания": "PendingDeprecationWarning",
    "ПредупреждениеУстаревания": "DeprecationWarning",
    "ПредупреждениеРесурса": "ResourceWarning",
    "ПредупреждениеВыполнения": "RuntimeWarning",
    "ПредупреждениеСинтаксиса": "SyntaxWarning",
    "ПредупреждениеПользователя": "UserWarning",
    "ПредупреждениеБудущего": "FutureWarning",
    "ПредупреждениеПодключения": "ImportWarning",
    "ПредупреждениеЮникода": "UnicodeWarning",
    "ПредупреждениеБайтов": "BytesWarning",
    "БазовоеИсключение": "BaseException",
    "Исключение": "Exception",
    "СистемныйВыход": "SystemExit",
    "ОстановкаИнтерации": "StopIteration",
    "НеРеализовано": "NotImplemented",
    "КлавиатурноеПрерывание": "KeyboardInterrupt",
    "ВыходГенератора": "GeneratorExit",
}

methods = {
    "заменить": "replace",
    "соединить": "join",
    "элементы": "items",
    "расширить": "extend",
    "добавить": "append",
    "вставить": "insert",
    "хлопнуть": "pop",
    "есть_алфчисл": "isalnum",
    "есть_алфавитный": "isalpha",
    "есть_ascii": "isascii",
    "есть_десятичный": "isdecimal",
    "есть_цифровой": "isdigit",
    "есть_идентификатор": "isidentifier",
    "есть_числовой": "isnumeric",
    "есть_печатный": "isprintable",
    "есть_пробел": "isspace",
    "есть_заголовок": "istitle",
    "есть_верхний": "isupper",
    "есть_нижний": "islower",
    "читать": "read",
    "писать": "write",
    "начинаетсяс": "startswith",
    "заканчиваетсяна": "endswith",
    "нижний": "lower",
    "верхний": "upper",
    "индекс": "index",
    "обновить": "update",
    "ключи": "keys",
    "значения": "values",
    "разделить": "split",
    "лразделить": "lsplit",
    "празделить": "rsplit",
    "обрезать": "strip",
    "лобрезать": "lstrip",
    "побрезать": "rstrip",
    "поменятьрегистр": "swapcase",
    "как_целая_пропорция": "as_integer_ratio",
    "мнимое": "imag",
    "действительное": "real",
    "копировать": "copy",
    "очистить": "clear",
    "хлопнутьэлемент": "popitem",
    "удалить": "remove",
    "количество": "count",
    "развернуть": "reverse",
    "сортировать": "sort",
    "получить": "get"
}

other_names = {
    "сам": "self",
    "клс": "cls",
    "значение": "value",
    "ключ": "key",
    "атрибут": "attr",
    "атрибуты": "attributes",
    "токены": "tokens",
    "токен": "token",
    "код": "code",
    "описание": "description",
    "результат": "result",
    "аргумент": "arg",
    "аргументы": "args",
    "файл": "file",
    "время": "time",
    "датавремя": "datetime",
    "дата": "date",
    "данные": "data",
    "ответ": "response",
    "запрос": "request",
    "ошибка": "error",
    "имя": "name",
    "элемент": "item",
    "состояние": "state",
    "проверить": "check",
    "статус": "status",
    "текст": "text",
    "преобразовать": "transform",
    "парсить": "parse",
    "парсер": "parser",
    "приятный": "pretty",
    "дерево": "tree",
    "число": "number",
    "граница": "bound",
    "главный": "main",
    "слово": "word",
    "слова": "words",
}

translated = {
    **types,
    **magic,
    **builtin_functions,
    **methods,
    **errors,
    **other_names,
}



from ..translated import translated as переводы
from ..argparser import аргпарсер
import os
from ..codegen_utils import БазовыйГенератор, получить_парсеры, загрузить, transform

здесь = os.path.abspath(os.path.dirname(__file__))
(
    parser,
    вычислительный_парсер,
) = получить_парсеры(os.path.join(здесь, "birp.lark"))


class Кодогенератор(БазовыйГенератор):
    вычислительный_парсер = вычислительный_парсер
    переводы = переводы


преобразователь = Кодогенератор()


def main():
    args = аргпарсер.parse_args()
    if args.переводы:
        with open(args.переводы) as file:
            переводы_пользователя = загрузить(file)

        переводы.update(переводы_пользователя)
    if args.файлы:
        try:
            import black as чёрный
        except ImportError:
            чёрный = None

        for arg in args.файлы:
            имя_ввода = arg
            with open(имя_ввода) as file:
                code = file.read()

            имя_вывода = имя_ввода.replace(".birp", ".py")
            (
                result,
                tree,
            ) = transform(code, parser, преобразователь, имя_ввода)
            if result:
                if чёрный:
                    result = чёрный.format_str(result, mode=чёрный.Mode())
                if args.tree:
                    with open(имя_вывода + ".tree", "w") as file:
                        file.write(tree.pretty())

                with open(имя_вывода, "w") as file:
                    file.write(result)

                print(f"{имя_ввода} -> {имя_вывода}")
            else:
                print(f"{имя_ввода} невозможно преобразовать в {имя_вывода}")
    else:
        while True:
            print("Интерактивный Борп!")
            code = input(">>> ")
            (
                result,
                tree,
            ) = transform(code, вычислительный_парсер, преобразователь, "<ввод>")
            print(f"<<< {result}")
            print(f"= {repr(eval(result))}")


if __name__ == "__main__":
    main()



from lark import (
    Lark as Жаворонок,
    Transformer as Преобразователь,
    UnexpectedToken as НепредвиденныйТокен,
    UnexpectedCharacters as НепредвиденныеСимволы,
    UnexpectedEOF as НепредвиденныйКФ,
)
from lark.indenter import Indenter as Отступник
from json import load as загрузить
from traceback import format_exc


class ОтступникПитона(Отступник):
    NL_type = "_NEWLINE"
    OPEN_PAREN_types = ["LPAR", "LSQB", "LBRACE"]
    CLOSE_PAREN_types = ["RPAR", "RSQB", "RBRACE"]
    INDENT_type = "_INDENT"
    DEDENT_type = "_DEDENT"
    tab_len = 8


символ_отступа: str = "    "


def добавить_отступ(text: str) -> str:
    if text.strip():
        return text.replace("\n", "\n" + символ_отступа)
    return ""


def с_пробелом(text):
    return " ".join(["", text])


def пропустить_пустые(tokens):
    return filter(None, tokens)


def проверка_интерполяции(token, курсор):
    вложенность = 0
    for смещение in range(курсор, len(token)):
        if token[смещение] == "{":
            вложенность += 1
        elif token[смещение] == "}":
            вложенность -= 1
        if вложенность < 0:
            return смещение
    else:
        raise ValueError("Неверная интерполяция")


def получить_парсеры(файл_грамматики):
    with open(файл_грамматики) as file:
        грамматика = file.read()

    args = dict(parser="lalr", postlex=ОтступникПитона(), maybe_placeholders=True)
    parser = Жаворонок(грамматика, start="file_input", **args)
    вычислительный_парсер = Жаворонок(грамматика, start="eval_input", **args)
    return parser, вычислительный_парсер


class БазовыйГенератор(Преобразователь):
    keys = {
        "if": "if",
        "else": "else",
        "elif": "elif",
        "while": "while",
        "for": "for",
        "try": "try",
        "except": "except",
        "finally": "finally",
        "from": "from",
        "in": "in",
        "import": "import",
        "as": "as",
        "pass": "pass",
        "continue": "continue",
        "break": "break",
        "return": "return",
        "def": "def",
        "raise": "raise",
        "assert": "assert",
        "class": "class",
        "yield": "yield",
        "await": "await",
        "async": "async",
        "lambda": "lambda",
        "None": "None",
        "True": "True",
        "False": "False",
        "and": "and",
        "or": "or",
        "not": "not",
        "is": "is",
        "del": "del",
        "match": "match",
        "case": "case",
        "global": "global",
        "nonlocal": "nonlocal",
        "with": "with",
        "orig_in": "в",
        "orig_is": "есть",
        "orig_not": "не",
    }

    def file_input(self, tokens):
        return "\n".join(tokens)

    def eval_input(self, tokens):
        return tokens[0]

    def __getattr__(self, attr):
        if attr.isupper():
            return lambda token: token
        print(f"!!! {attr} не преобразован !!!")
        return lambda tokens: " ".join(tokens)

    def one_line_suite(self, tokens):
        return tokens[0]

    def suite(self, tokens):
        return добавить_отступ("\n" + "\n".join(tokens))

    def if_stmt(self, tokens):
        иначе_ = (f"\n{self.keys['else']}:" + tokens[3]) if tokens[3] else ""
        return "".join(
            [
                self.keys["if"],
                " ",
                tokens[0],
                ":",
                tokens[1],
                *(["\n", tokens[2]] if tokens[2] else ""),
                иначе_,
            ]
        )

    def elifs(self, tokens):
        return "\n".join(tokens)

    def elif_(self, tokens):
        return "".join([self.keys["elif"], " ", tokens[0], ":", tokens[1]])

    def import_from(self, tokens):
        return " ".join(
            [self.keys["from"], "".join(tokens[:-1]), self.keys["import"], tokens[-1]]
        )

    def import_name(self, tokens):
        return " ".join([self.keys["import"], tokens[0]])

    def dotted_as_name(self, tokens):
        return f" {self.keys['as']} ".join(пропустить_пустые(tokens))

    def import_as_name(self, tokens):
        return f" {self.keys['as']} ".join(пропустить_пустые(tokens))

    def dotted_as_names(self, tokens):
        return ", ".join(tokens)

    def import_as_names(self, tokens):
        return ", ".join(tokens)

    def dotted_name(self, tokens):
        return ".".join(tokens)

    def dots(self, tokens):
        return "".join(tokens)

    def global_stmt(self, tokens):
        return " ".join([self.keys["global"], ", ".join(tokens)])

    def nonlocal_stmt(self, tokens):
        return " ".join([self.keys["nonlocal"], ", ".join(tokens)])

    def assert_stmt(self, tokens):
        return " ".join([self.keys["assert"], ", ".join(пропустить_пустые(tokens))])

    def raise_stmt(self, tokens):
        return " ".join(
            [
                self.keys["raise"],
                f" {self.keys['from']} ".join(пропустить_пустые(tokens)),
            ]
        )

    def while_stmt(self, tokens):
        иначе_ = (f"\n{self.keys['else']}:" + tokens[2]) if tokens[2] else ""
        return "".join([self.keys["while"], " ", tokens[0], ":", tokens[1], иначе_])

    def continue_stmt(self, tokens):
        return self.keys["continue"]

    def break_stmt(self, tokens):
        return self.keys["break"]

    def assign(self, tokens):
        return " = ".join(tokens)

    def slice(self, tokens):
        tokens = [x or "" for x in tokens]
        return "{}:{}{}".format(*tokens)

    def sliceop(self, tokens):
        if tokens[0]:
            return f":{tokens[0]}"

    def var(self, tokens):
        return tokens[0]

    def test(self, tokens):
        return "{0} {3} {1} {4} {2}".format(*tokens, self.keys["if"], self.keys["else"])

    def star_expr(self, tokens):
        return f"*{tokens[0]}"

    def funccall(self, tokens):
        return f"{tokens[0]}({tokens[1] or ''})"

    def arguments(self, tokens):
        return ", ".join(пропустить_пустые(tokens))

    def argvalue(self, tokens):
        return "=".join(tokens)

    def starargs(self, tokens):
        return ", ".join(пропустить_пустые(tokens))

    def stararg(self, tokens):
        return f"*{tokens[0]}"

    def kwargs(self, tokens):
        return f"**{tokens[0]}"

    def subscript_tuple(self, tokens):
        return ", ".join(tokens)

    def parens(self, tokens):
        return f"({tokens[0]})"

    def getitem(self, tokens):
        return f"{tokens[0]}[{tokens[1]}]"

    def getattr(self, tokens):
        return ".".join(tokens)

    def name(self, tokens):
        return self.переводы.get(tokens[0], tokens[0])

    def annassign(self, tokens):
        if tokens[2]:
            return f"{tokens[0]}: {tokens[1]} = {tokens[2]}"
        return f"{tokens[0]}: {tokens[1]}"

    def augassign(self, tokens):
        return " ".join(tokens)

    def augassign_op(self, tokens):
        return tokens[0]

    def float(self, tokens):
        return tokens[0]

    def integer(self, tokens):
        return tokens[0]

    def complex(self, tokens):
        return tokens[0]

    def string(self, tokens):
        return tokens[0]

    def with_stmt(self, tokens):
        return "{2} {0}:{1}\n".format(*tokens, self.keys["with"])

    def with_items(self, tokens):
        return ", ".join(tokens)

    def with_item(self, tokens):
        return f" {self.keys['as']} ".join(пропустить_пустые(tokens))

    def STRING(self, token):
        if token[0] in "юбфрЮБФР":
            token = "ubfrUBFR"["юбфрЮБФР".index(token[0])] + token[1:]
        if token.lower().startswith("f"):
            result = []
            интерполяции = [0]
            скобка = False
            index = 0
            while index < len(token):
                if скобка:
                    скобка = False
                    index += 1
                    continue
                if token[index] == "{":
                    if token[index + 1 : index + 2] == "{":
                        скобка = True
                        continue
                    if not скобка:
                        интерполяции.append(index + 1)
                        index = проверка_интерполяции(token, index + 1)
                        интерполяции.append(index)
                скобка = False
                index += 1
            интерполяции.append(len(token))
            for index in range(len(интерполяции[:-1])):
                часть = token[интерполяции[index] : интерполяции[index + 1]]
                if index % 2:
                    часть_ = часть.split(":")
                    if len(часть_) > 1:
                        (
                            *часть,
                            фмт,
                        ) = часть_
                    else:
                        часть = часть_
                        фмт = None
                    часть = self.transform(
                        self.вычислительный_парсер.parse(":".join(часть))
                    )
                    result.append(":".join(пропустить_пустые([часть, фмт])))
                else:
                    result.append(часть)
            return "".join(result)
        return token

    def funcdef(self, tokens):
        return f"\n{self.keys['def']} {tokens[0]}({tokens[1] or ''}){' -> '.join(['', *пропустить_пустые([tokens[2]])])}: {tokens[3]}"

    def parameters(self, tokens):
        return ", ".join(пропустить_пустые(tokens))

    def paramvalue(self, tokens):
        return " = ".join(tokens)

    def const_true(self, tokens):
        return self.keys["True"]

    def const_false(self, tokens):
        return self.keys["False"]

    def const_none(self, tokens):
        return self.keys["None"]

    def starparams(self, tokens):
        return "".join(["*", ", ".join(пропустить_пустые(tokens))])

    def kwparams(self, tokens):
        return f"**{tokens[0]}"

    def typedparam(self, tokens):
        return ": ".join(tokens)

    def return_stmt(self, tokens):
        return " ".join([self.keys["return"], *пропустить_пустые(tokens)])

    def lambdef(self, tokens):
        return "".join(
            [self.keys["lambda"], с_пробелом(tokens[0] or ""), ": ", tokens[1]]
        )

    def lambda_params(self, tokens):
        return ", ".join(пропустить_пустые(tokens))

    def lambda_paramvalue(self, tokens):
        return "=".join(пропустить_пустые(tokens))

    def lambda_starparams(self, tokens):
        return ", ".join(пропустить_пустые(tokens))

    def lambda_kwparams(self, tokens):
        return f"**{tokens[0]}"

    def or_test(self, tokens):
        return f" {self.keys['or']} ".join(tokens)

    def and_test(self, tokens):
        return f" {self.keys['and']} ".join(tokens)

    def not_test(self, tokens):
        return f"{self.keys['not']} {tokens[0]}"

    def or_expr(self, tokens):
        return " | ".join(tokens)

    def xor_expr(self, tokens):
        return " ^ ".join(tokens)

    def and_expr(self, tokens):
        return " & ".join(tokens)

    def power(self, tokens):
        return " ** ".join(tokens)

    def comparison(self, tokens):
        return " ".join(tokens)

    def comp_op(self, tokens):
        return (
            tokens[0]
            .replace(self.keys["orig_not"], self.keys["not"])
            .replace(self.keys["orig_is"], self.keys["is"])
            .replace(self.keys["orig_in"], self.keys["in"])
        )

    def многострочный_список(self, tokens):
        return добавить_отступ("\n" + ",\n".join(tokens))

    def list(self, tokens):
        if len(tokens) > 5 or any("\n" in token for token in tokens):
            return "".join(["[", self.многострочный_список(tokens), "\n]"])
        return f"[{', '.join(tokens)}]"

    def dict(self, tokens):
        if len(tokens) > 5 or any("\n" in token for token in tokens):
            return "".join(["{", self.многострочный_список(tokens), "\n}"])
        return f"{{{', '.join(tokens)}}}"

    def set(self, tokens):
        if len(tokens) > 5 or any("\n" in token for token in tokens):
            return "".join(["{", self.многострочный_список(tokens), "\n}"])
        return f"{{{', '.join(tokens)}}}"

    def dict_comprehension(self, tokens):
        return f"{{{tokens[0]}}}"

    def shift_expr(self, tokens):
        return " ".join(tokens)

    def set_comprehension(self, tokens):
        return f"{{{tokens[0]}}}"

    def list_comprehension(self, tokens):
        return f"[{tokens[0]}]"

    def classdef(self, tokens):
        return "".join(
            [
                f"\n{self.keys['class']} ",
                tokens[0],
                f"({tokens[1]})" if tokens[1] else "",
                ":",
                tokens[2],
                "\n",
            ]
        )

    def comprehension(self, tokens):
        return " ".join(пропустить_пустые(tokens))

    def comp_fors(self, tokens):
        return " ".join(tokens)

    def comp_for(self, tokens):
        return " ".join(
            [
                *пропустить_пустые([f"{self.keys['async']} " if tokens[0] else ""]),
                self.keys["for"],
                tokens[1],
                self.keys["in"],
                tokens[2],
            ]
        )

    def comp_if(self, tokens):
        return f"{self.keys['if']} {tokens[0]}"

    def exprlist(self, tokens):
        return ", ".join(tokens)

    def yield_expr(self, tokens):
        return " ".join([self.keys["yield"], *пропустить_пустые(tokens)])

    def yield_from(self, tokens):
        return " ".join([self.keys["yield"], self.keys["from"], tokens[0]])

    def arith_expr(self, tokens):
        return " ".join(пропустить_пустые(tokens))

    def key_value(self, tokens):
        return ": ".join(tokens)

    def decorator(self, tokens):
        return "".join(["@", tokens[0], f"({tokens[1]})" if tokens[1] else ""])

    def decorators(self, tokens):
        return "\n".join(tokens)

    def decorated(self, tokens):
        return "".join(["\n", *tokens])

    def tuple(self, tokens):
        if tokens:
            if len(tokens) > 5 or all("\n" in token for token in tokens):
                return "".join(["(", self.многострочный_список(tokens), ",\n)"])
            return f"({', '.join(tokens)}, )"
        return "()"

    def testlist_tuple(self, tokens):
        return ", ".join(tokens)

    def simple_stmt(self, tokens):
        return "; ".join(tokens)

    def for_stmt(self, tokens):
        else_ = (f"\n{self.keys['else']}:" + tokens[3]) if tokens[3] else ""
        return "".join(
            [
                f"{self.keys['for']} ",
                tokens[0],
                f" {self.keys['in']} ",
                tokens[1],
                ":",
                tokens[2],
                else_,
            ]
        )

    def term(self, tokens):
        return " ".join(tokens)

    def factor(self, tokens):
        return " ".join(tokens)

    def async_funcdef(self, tokens):
        return " ".join([f"\n{self.keys['async']}", tokens[0][1:]])

    def del_stmt(self, tokens):
        return f"{self.keys['del']} {tokens[0]}"

    def await_expr(self, tokens):
        return f"{self.keys['await']} {tokens[1]}"

    def try_stmt(self, tokens):
        else_ = (f"{self.keys['else']}:" + tokens[2]) if tokens[2] else ""
        return "\n".join(
            пропустить_пустые(
                [
                    f"{self.keys['try']}: {tokens[0]}",
                    tokens[1],
                    else_ if tokens[2] else "",
                    tokens[3],
                    " ",
                ]
            )
        )

    def try_finally(self, tokens):
        return f"\n{self.keys['try']}: {tokens[0]}\n{tokens[1]}"

    def except_clauses(self, tokens):
        return "\n".join(tokens)

    def except_clause(self, tokens):
        if len(tokens) == 2:
            tokens.insert(1, None)
        return "".join(
            [
                self.keys["except"],
                с_пробелом(
                    f" {self.keys['as']} ".join(пропустить_пустые(tokens[:2]))
                ).rstrip(),
                ":",
                tokens[2],
            ]
        )

    def finally_(self, tokens):
        return f"{self.keys['finally']}: {tokens[0]}"

    def du_dct(self, tokens):
        return f"**{tokens[0]}"

    def pass_stmt(self, tokens):
        return self.keys["pass"]

    def async_stmt(self, tokens):
        nl = "\n"
        return f"{self.keys['async']} {tokens[0].lstrip(nl)}"

    def match_stmt(self, tokens):
        ветки = добавить_отступ("\n" + "\n".join(tokens[1:]))
        return f"{self.keys['match']} {tokens[0]}:{ветки}"

    def case(self, tokens):
        (
            шаблон,
            условие,
            блок,
        ) = tokens
        if условие:
            условие = f" {self.keys['if']} {условие}"
        else:
            условие = ""
        блок = добавить_отступ(f"\n{блок}")
        return f"{self.keys['case']} {шаблон}{условие}:{блок}"

    def sequence_pattern(self, tokens):
        return f"({', '.join(tokens)})"

    def as_pattern(self, tokens):
        return f"{tokens[0]} {self.keys['as']} {tokens[1]}"

    def or_pattern(self, tokens):
        return " | ".join(tokens)

    def literal_pattern(self, tokens):
        return tokens[0]

    def mapping_pattern(self, tokens):
        return f"{{{', '.join(tokens)}}}"

    def mapping_pattern_item(self, tokens):
        return ":".join(tokens)

    def star_pattern(self, tokens):
        return f"*{tokens}"

    def mapping_star_pattern(self, tokens):
        if len(tokens) == 1:
            tokens.insert(0, "")
        return f"{{{' ** '.join(tokens)}}}"

    def attr_pattern(self, tokens):
        return ".".join(tokens)

    def pos_arg_pattern(self, tokens):
        return ", ".join(tokens)

    def keyws_arg_pattern(self, tokens):
        return ", ".join(tokens)

    def keyw_arg_pattern(self, tokens):
        return " = ".join(tokens)

    def arguments_pattern(self, tokens):
        return ", ".join(tokens)

    def no_pos_arguments(self, tokens):
        return tokens[0]


def репорт(code, error, имя_файла="модуль"):
    номер_строки = error.line
    размер = 3
    верхний_размер = min(номер_строки - 1, размер)
    строки = code.split("\n")[номер_строки - верхний_размер - 1 : номер_строки + размер]
    отступ = error.column
    ширина_номера_строки = len(
        str(max(номер_строки - верхний_размер, номер_строки + размер))
    )
    return "\n".join(
        [
            *[
                f"{str(i + номер_строки - верхний_размер):>{ширина_номера_строки}} | {line}"
                for i, line in enumerate(строки[: верхний_размер + 1])
            ],
            f"{'':>{ширина_номера_строки + 1}}:{'':>{отступ}}^ Ошибка: Непредвиденный {error.token.type}",
            *[
                f"{str(i + 1 + номер_строки):>{ширина_номера_строки}} | {line}"
                for i, line in enumerate(строки[верхний_размер + 1 :])
            ],
        ]
    )


def transform(code, parser, преобразователь, имя_файла="модуль"):
    try:
        tree = parser.parse(code + "\n")
    except НепредвиденныйТокен as error:
        print(репорт(code, error, имя_файла))
        return None, None

    try:
        result = преобразователь.transform(tree)
    except Exception:
        print(format_exc())
        return None, tree

    return result, tree
